/*============================================================================
         SPANNING FOREST of a non necessarily connected, DIRECTED graph.
1. Nodes are denoted by integers: 0,...,N-1.
2. The graph is represented as an array G of lists of nodes. In G[i] there is
   a list L of nodes. Node m occurs in L iff there is an arc from i to m.
   An arc (n,m) should have n different from m.
3. The boolean array visited[0..N-1] is such that visited[i]=true iff node i
   has been visited.
4. arcs is a list of pairs of nodes. (n,m) is in arcs iff (n,m) is an arc of
   the spanning forest.
The input graph is given in the file graph2_in whose content is the sequence
of integers:
0 1 0 2 0 3 0 4 0 5 1 2 3 1 5 4 meaning that there is an arc from 0 to 1, an
arc from 0 to 2, etc. A different graph is in the file graph3nc_in whose
content is the sequence: 0 5 4 0 3 1 5 4
=========================================================================== */
#include <fstream.h>  // file for using files
fstream input_graph;  // file for the input_graph
//----------------------------------------------------------------------------
typedef int node;     // nodes are integers
const int N=6;        // N is the number of nodes in the graph. N>=2
//----------------------------------------------------------------------------
//                    pair is (node1,node2)
class pair
{node f, s;
 public :
 pair(node a, node b) {f=a; s=b;}
 node fst(){return f;}
 node snd(){return s;}
};
//----------------------------------------------------------------------------
//                    pair_list is (node1,node2) list
class pair_list
{struct pri_cell                                        // pri_cell is private
   {pair atom; pri_cell* next;
    pri_cell(pair at, pri_cell* nx) : atom(at), next(nx) { };};
    pri_cell* ptr;
public :
pair_list(): ptr(NULL){}

bool isnull() {return (ptr == NULL);}

pair phead()
{if (ptr == NULL) {cout << "\nError 1: head of empty_list!\n"; }
 else {return ptr->atom;} }

void ptail()
{if (ptr == NULL) {cout << "\nError 2: tail of empty_list!\n"; }
 else {ptr=ptr->next;} }

void pcons(pair x) {ptr=new pri_cell(x,ptr);}

void printpair_list()
{if (ptr == NULL) {cout << "nil";}
 else {pair ip=ptr->atom; cout <<"("<<ip.fst()<<","<<ip.snd()<<") ";
       pri_cell* ptr1=ptr; ptr=ptr->next;
       if (ptr!=NULL) {printpair_list();}; ptr=ptr1;}}
};     // end of the class pair_list
//----------------------------------------------------------------------------
//                    node_list
class cell
{public:
  node atom; cell* next;
  cell(node at, cell* nx) : atom(at), next(nx) { };          // cell is public
};
typedef cell* node_list;          // node_list is a pointer to the public cell
//----------------------------------------------------------------------------
// The graph denoted by the global array G of N elements. Each element
// G[0],...,G[N-1} is a node_list
node_list G[N]={};
//----------------------------------------------------------------------------
node nhead(node_list L)
{if (L == NULL) {cout << "\nError 3: head of empty_list!\n";}
 else {return L->atom;}}

node_list ntail(node_list L)
{if (L == NULL) {cout << "\nError 4: tail of empty_list!\n";}
 else return L->next;}

node_list ncons(node n, node m) {node_list L=new cell(n,G[m]); return L;}

void printnode_list(node_list L)
{if (L == NULL) {cout << "nil";}
 else {node p=L->atom; cout << p <<" ";
       cell* L1=L; L=L->next; if (L!=NULL) {printnode_list(L);}; L=L1;}}
//----------------------------------------------------------------------------
void readgraph()
{int from_node=0, to_node=0;
for (int i=0; i<N-1; i=i+1) {G[i]=NULL;};
input_graph >> from_node;        // <-- eof()=1 if we read a non existing item
while (!input_graph.eof())
     {input_graph >> to_node;
      G[from_node]=ncons(to_node,from_node);
      input_graph >> from_node;      // <-- trying to read the first node
      };}                            //     of a new <from_node, to_node> pair
//----------------------------------------------------------------------------
//                    arcs of the spanning forest: a pair_list
pair_list arcs;
//----------------------------------------------------------------------------
//                    the global array `visited'. Initialized to false
bool visited[N]={};
//----------------------------------------------------------------------------
void visit(node n)
{visited[n]=true;
 while (G[n]!=NULL)
    {node m=nhead(G[n]);
     if (visited[m]==false)
      {arcs.pcons(pair(n,m));   // adding to the list arcs the arc from n to m
       visit(m);};
     G[n]=ntail(G[n]);};        // the global array G is modified
};
//----------------------------------------------------------------------------
void main()
{cout << "\nThe nodes are: {0,...,"<<N-1<<"}.\n";
input_graph.open("graph2_in", ios::in);
// graph2_in   is directed and connected
// graph3nc_in is directed and not connected
// graph4nc_in is directed and not connected
readgraph();
cout << "\nThe given directed graph represented as array of lists, is:";
for (int i=0; i<N; i++)
{cout << "\n G["<<i<<"]= "; printnode_list(G[i]); }; cout << "\n";
//----------------------------------------------------------------------------
// The for statement is required because the graph may be not connected.
for (int i=0; i<N; i++)
{if (visited[i]==false && G[i]!=NULL)
 {visit(i);};}
cout << "\nThe directed arcs of a spanning forest are:\n";
arcs.printpair_list(); cout << "\n\n";
//----------------------------------------------------------------------------
// printing the single_node trees of the spanning forest.
bool multiple_node_trees[N]={};
while (!arcs.isnull()) //
{multiple_node_trees[(arcs.phead()).fst()]=true;
 multiple_node_trees[(arcs.phead()).snd()]=true;
 arcs.ptail();};
cout << "The single_node trees in the spanning forest are:\n-";
for (int i=0; i<N; i++)
{if (multiple_node_trees[i]==false) {cout << " " << i;};};
cout << " -\n\n";
}
/*----------------------------------------------------------------------------
For the graph2_in the output is:
    The nodes are: {0,...,5}.

    The given directed graph represented as
    array of lists, is:
     G[0]= 5 4 3 2 1
     G[1]= 2
     G[2]= nil
     G[3]= 1
     G[4]= nil
     G[5]= 4

    The directed arcs of a spanning forest are:
    (1,2) (3,1) (0,3) (5,4) (0,5)

    The single_node trees in the spanning forest are:
    - -
------------------------------------------------------------------------------
For the graph3nc_in the output is:
    The nodes are: {0,...,5}.

    The given directed graph represented as array of lists, is:
      G[0]= 5
      G[1]= nil
      G[2]= nil
      G[3]= 1
      G[4]= 0
      G[5]= 4

    The directed arcs of a spanning forest are:
    (3,1) (5,4) (0,5)

    The single_node trees in the spanning forest are:
    - 2 -
------------------------------------------------------------------------------
For the graph4nc_in the output is:
    The nodes are: {0,...,5}.

    The given directed graph represented as array of lists, is:
      G[0]= 4
      G[1]= 3
      G[2]= 1
      G[3]= 2
      G[4]= 5
      G[5]= nil

    The directed arcs of a spanning forest are:
    (3,2) (1,3) (4,5) (0,4)

    The single_node trees in the spanning forest are:
    - -
--------------------------------------------------------------------------- */